/*!
*	\addtogroup Modulo4
*	\{
*/
/*!
*	\addtogroup TestModulo4 Specifiche per il Test
*	\{
*/

#include "LPC_Include.h"
#include <stdio.h>
#include <stdlib.h>

#define NUMFIELDS 10
#define NUMVALUES 5
static int handle = -1;
static FIELD_VALUE_t FieldValue[NUMFIELDS];
static FIELD_VALUE_t FieldValueAll[NUMFIELDS];
static FIELD_VALUE_t Record20[NUMFIELDS];
static FIELD_VALUE_t Record30[NUMFIELDS];
static FIELD_DATA_t FieldData[NUMFIELDS];
static char Appo[256];
static char *Nome, *Sesso, *Comune, *Provincia, *Telefono, *Indirizzo, *Studente;
static	time_t *Data;
static	double *Eta;
static	double *Reddito;

void TestEdit(void); 		//!< Funzione di test per la EditRecord()
void TestRead(void);   		//!< Funzione di test per la ReadRecord()
void TestLocate(void); 		//!< Funzione di test per la LocateRecord()
void TestMisto(void);  		//!< Funzione di test misto per tutte le funzioni
void Result(char *Func);	//!< Funzione di calcolo e stampa risultati
int ConfrontaFiles(char *File1, char *File2); //!< Funzione di confronto files
int Startup(int handle, int reset);	//!< Funzione di inizializzazione
void Quit(void);
int ConfrontaValues(FIELD_VALUE_t *FieldValue, FIELD_VALUE_t *FieldValueRef, int NumValues);

// Vettori per gli id dei test, il check e i pesi relativi
#define MAX_TEST 100
typedef struct {
	int testid;
	int weight;
	char desc[256];
	int passed;
} TEST_t;
		
static int numtest;
static TEST_t test[MAX_TEST];

/*!
*	\brief Test per le funzioni di interfaccia del Modulo 4
*
* Programma di test per il Modulo 4
*
* Ogni test e' identificato da un numero cosi' composto:\n
* \p xyynn \n
*  x	Id del modulo\n
* yy	Id della funzione\n
* nn	Id del test\n
*
* Ogni test ha anche un peso che viene indicato nella stampa dei risultati.
* I test di funzionalit hanno i pesi piu' elevati.
* In questo test viene verificato che l'errore ritornato sia effettivamente 
* quello indicato dalla funzione.
*
* La funzione di test puo' essere chiamata anche da linea di comando 
* con: TestM4 opzione startup
* Dove opzione e' il numero dell'opzione scelta e startup e' 1 o 0 a seconda
* che si voglia reinizializzare o meno il file dbf.
*
* Vengono effettuati i seguenti controlli: 
* - Edit (EditRecord()): 
*   - Handle del DBF errato
*   - NumValues errato
*   - FieldValue o uno dei campi errato
*   - Verifica funzionalita' 
*
* - Lettura (ReadRecord()):
*   - Handle del DBF errato
*   - NumValues errato
*   - FieldValue o uno dei campi errato
*   - Verifica funzionalita' 
* 
* - Locate (LocateRecord()):
*   - Handle del DBF errato
*   - NumValues errato
*   - FieldValue o uno dei campi errato
*   - Verifica funzionalita' 
*
* - Misto
*   - Modifica di piu' record in varie posizioni, spostamenti, letture e locate
****************************************************/


int main(int argc, char **argv)
{
	char scelta[10];
	int sel, reset;

	printf("Programma di Test per il Modulo 4\n\n");

	// Vedo se mi e' stato chiesta l'inizializzazione completa
	if (argc == 3)
		reset = atoi(argv[2]);
	else
		if (argc < 2)
			reset = 1;
		else
			reset = 0;

	// Pulizia iniziale e preparazione dell'handle e allocazione
	handle = Startup(handle, reset);
	if (handle < 0)
	{
		printf("TEST: Errore apertura database:[%d]\n", handle);
		printf("Exiting....\n");
		return(-1);
	}

	for(sel = 0;;)
	{
		if (argc < 2)
		{
			printf("\t1. Test funzione EditRecord\n");
			printf("\t2. Test funzione ReadRecord\n");
			printf("\t3. Test funzione LocateRecord\n");
			printf("\t4. Test funzione Misto\n");
			printf("\t5. Test completo e valutazione\n");
			printf("\n\t0. Uscita\n");
			printf("\n\nScelta: ");

			(void) fgets(scelta, sizeof(scelta), stdin);
			sel = atoi(scelta);
		}
		else // Faccio solo 1 giro
		{
			if (sel)
				sel = 0;
			else
				sel = atoi(argv[1]);
		}

		Result(NULL);
		switch(sel)
		{
		case 0:
			Quit();
			if (argc < 2)
				printf("Bye!\n\n");
			return(0);
		case 1:
			TestEdit();
			Result("EDITRECORD");	
			break;
		case 2:
			TestRead();
			Result("READRECORD");	
			break;
		case 3:
			TestLocate();
			Result("LOCATERECORD");	
			break;
		case 4:
			TestMisto();
			Result("MISTO");	
			break;
		case 5:
			// Devo ripulire prima di ricominciare
			handle = Startup(handle, 1);
			if (handle < 0)
			{
				printf("TEST: Errore apertura database:[%d]\n", handle);
				printf("Exiting....\n");
				return(-1);
			}
			TestEdit();
			TestRead();
			TestLocate();
			TestMisto();
			Result("FINALE");
			break;

		default:
			printf("Scelta errata...\n");
			break;;
		}
	}
}
void TestEdit()
{
	int ret= 0;
	DBF_INFO_t DBFInfo;
	char *p;
	time_t Start, Stop;

	printf("Test EDIT\n");

// Test sull'handle
	test[numtest].testid = 40101;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "EDIT: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  EditRecord(handle+10, NUMVALUES, &FieldValue[0]);
	printf("Ritorno della EditRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul NumValues (1)
	test[numtest].testid = 40102;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "EDIT: NumValues errato (1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  EditRecord(handle, 0, &FieldValue[0]);
	printf("Ritorno della EditRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul NumValues (2)
	test[numtest].testid = 40103;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "EDIT: NumValues errato(2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  EditRecord(handle, 100, &FieldValue[0]);
	printf("Ritorno della EditRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue (1)
	test[numtest].testid = 40104;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "EDIT: FieldValue errato (1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  EditRecord(handle, NUMVALUES, NULL);
	printf("Ritorno della EditRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue (2)
	test[numtest].testid = 40105;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "EDIT: FieldValue errato (2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	strcpy(Appo, FieldValue[4].Name);
	strcpy(FieldValue[4].Name, "COGNOME");
	ret =  EditRecord(handle, NUMVALUES, &FieldValue[0]);
	printf("Ritorno della EditRecord: [%d]\n", ret);
	if (ret == LPC_BAD_FIELD)
		test[numtest].passed = 1;
	strcpy(FieldValue[4].Name, Appo);
	numtest++;

// Test sul FieldValue (3)
	test[numtest].testid = 40106;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "EDIT: FieldValue errato (3)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	p = FieldValue[3].value;
	FieldValue[3].value = NULL;
	ret =  EditRecord(handle, NUMVALUES, &FieldValue[0]);
	printf("Ritorno della EditRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	FieldValue[3].value = p;
	numtest++;

// Verifica funzionalita' (1)
	test[numtest].testid = 40107;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "EDIT: Funzionalita'(1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = GotoRecord(handle, ABSOLUTE, 1000);
		if (ret != LPC_ERR_EOF)
			break;

		ret =  EditRecord(handle, NUMVALUES, &FieldValue[0]);
		printf("Ritorno della EditRecord: [%d]\n", ret);
		if (ret != LPC_ERR_EOF)
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;

// Verifica funzionalita' (2)
	test[numtest].testid = 40108;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "EDIT: Funzionalita'(2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = GotoRecord(handle, ABSOLUTE, 50);
		if (ret != 50)
			break;

		time(&Start);
		// Modifico i primi NUMVALUES
		ret =  EditRecord(handle, NUMVALUES, &FieldValue[0]);
		printf("Ritorno della EditRecord: [%d]\n", ret);
		if (ret != LPC_OK)
			break;
		time(&Stop);

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		if ((DBFInfo.CurrentRec != 51) || (DBFInfo.LastUpdate > Stop) || (DBFInfo.LastUpdate < Start))
			break;

		if (ConfrontaFiles("Prova4.dbf", "ProvaTest1.dbf"))
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;


}

void TestRead()
{
	int i, ret= 0;
	DBF_INFO_t DBFInfo;
	char *p;
	time_t TimeStamp;

	printf("Test READ\n");

// Test sull'handle
	test[numtest].testid = 40201;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "READ: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  ReadRecord(handle+1, NUMVALUES, &FieldValue[0]);
	printf("Ritorno della ReadRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul NumValues (1)
	test[numtest].testid = 40202;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "READ: NumValues errato (1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  ReadRecord(handle, -1, &FieldValue[0]);
	printf("Ritorno della ReadRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul NumValues (2)
	test[numtest].testid = 40203;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "READ: NumValues errato(2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  ReadRecord(handle, 100, &FieldValue[0]);
	printf("Ritorno della ReadRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue (1)
	test[numtest].testid = 40204;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "READ: FieldValue errato (1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  ReadRecord(handle, NUMVALUES, NULL);
	printf("Ritorno della ReadRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue (2)
	test[numtest].testid = 40205;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "READ: FieldValue errato (2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	strcpy(Appo, FieldValue[4].Name);
	strcpy(FieldValue[4].Name, "COGNOME");
	ret =  ReadRecord(handle, NUMVALUES, &FieldValue[0]);
	printf("Ritorno della ReadRecord: [%d]\n", ret);
	if (ret == LPC_BAD_FIELD)
		test[numtest].passed = 1;
	strcpy(FieldValue[4].Name, Appo);
	numtest++;

// Test sul FieldValue (3)
	test[numtest].testid = 40206;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "READ: FieldValue errato (3)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	p = FieldValue[1].value;
	FieldValue[1].value = NULL;
	ret =  ReadRecord(handle, NUMVALUES, &FieldValue[0]);
	printf("Ritorno della ReadRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	FieldValue[1].value = p;
	numtest++;

// Verifica funzionalita' (1)
	test[numtest].testid = 40207;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "READ: Funzionalita'(1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = GotoRecord(handle, ABSOLUTE, 1000);
		if (ret != LPC_ERR_EOF)
			break;

		ret =  ReadRecord(handle, NUMVALUES, &FieldValue[0]);
		printf("Ritorno della ReadRecord: [%d]\n", ret);
		if (ret != LPC_ERR_EOF)
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;

// Verifica funzionalita' (2)
	test[numtest].testid = 40208;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "READ: Funzionalita'(2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = GotoRecord(handle, ABSOLUTE, 30);
		if (ret != 30)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		TimeStamp = DBFInfo.LastUpdate;

		for (i = 0; i < DBFInfo.NumFields; i++)
			strcpy(FieldValueAll[i].Name, "Hello");

		// Leggo Tutto
		ret =  ReadRecord(handle, 0, &FieldValueAll[0]);
		printf("Ritorno della ReadRecord: [%d]\n", ret);
		if (ret != LPC_OK)
			break;

		ret = LPC_OK;
		for (i = 0; i < DBFInfo.NumFields; i++)
		{
			if (strcmp( FieldValueAll[i].Name, FieldData[i].Name))
			{
				strcpy(FieldValueAll[i].Name, FieldData[i].Name);
				ret = -1;
			}
		}

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		if ((DBFInfo.CurrentRec != 31) || (DBFInfo.LastUpdate != TimeStamp))
			break;

		if (ConfrontaValues(&FieldValueAll[0], &Record30[0], NUMFIELDS))
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;

	// Verifica funzionalita' (3)
	test[numtest].testid = 40209;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "READ: Funzionalita'(2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = GotoRecord(handle, ABSOLUTE, 20);
		if (ret != 20)
			break;

		// Leggo un po' di campi
		ret =  ReadRecord(handle, NUMVALUES, &FieldValue[0]);
		printf("Ritorno della ReadRecord: [%d]\n", ret);
		if (ret != LPC_OK)
			break;

		if (ConfrontaValues(&FieldValue[0], &Record20[0], NUMVALUES))
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;

	// Verifica funzionalita' (4)
	test[numtest].testid = 40210;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "READ: Funzionalita'(3)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = GotoRecord(handle, RELATIVE, -1);
		if (ret != 20)
			break;

		// Leggo un solo campo
		ret =  ReadRecord(handle, 1, &FieldValue[9]);
		printf("Ritorno della ReadRecord: [%d]\n", ret);
		if (ret != LPC_OK)
			break;

		if (ConfrontaValues(&FieldValue[9], &Record20[9], 1))
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;

}

void TestLocate()
{
	int ret= 0;
	DBF_INFO_t DBFInfo;
	char Appo[256];
	char *p, c;
	time_t TimeStamp;

	printf("Test LOCATE\n");

// Test sull'handle
	test[numtest].testid = 40301;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "LOCATE: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  LocateRecord(handle+1, NUMVALUES, &FieldValue[0], TOP);
	printf("Ritorno della LocateRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul NumValues (1)
	test[numtest].testid = 40302;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "LOCATE: NumValues errato (1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  LocateRecord(handle, 0, &FieldValue[0], TOP);
	printf("Ritorno della LocateRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul NumValues (2)
	test[numtest].testid = 40303;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "LOCATE: NumValues errato(2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  LocateRecord(handle, 100, &FieldValue[0], TOP);
	printf("Ritorno della LocateRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue (1)
	test[numtest].testid = 40304;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "LOCATE: FieldValue errato (1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  LocateRecord(handle, NUMVALUES, NULL, TOP);
	printf("Ritorno della LocateRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue (2)
	test[numtest].testid = 40305;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "LOCATE: FieldValue errato (2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	strcpy(Appo, FieldValue[4].Name);
	strcpy(FieldValue[4].Name, "COGNOMEG");
	ret =  LocateRecord(handle, NUMVALUES, &FieldValue[0], TOP);
	printf("Ritorno della LocateRecord: [%d]\n", ret);
	if (ret == LPC_BAD_FIELD)
		test[numtest].passed = 1;
	strcpy(FieldValue[4].Name, Appo);
	numtest++;

// Test sul FieldValue (3)
	test[numtest].testid = 40306;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "LOCATE: FieldValue errato (3)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	p = FieldValue[3].value;
	FieldValue[3].value = NULL;
	ret =  LocateRecord(handle, NUMVALUES, &FieldValue[0], TOP);
	printf("Ritorno della LocateRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	FieldValue[3].value = p;
	numtest++;

// Test sul How
	test[numtest].testid = 40307;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "LOCATE: How errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  LocateRecord(handle, NUMVALUES, &FieldValue[0], 1000);
	printf("Ritorno della LocateRecord: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Verifica funzionalita' (1) non trovato current
	test[numtest].testid = 40308;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "LOCATE: Funzionalita'(1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	c = *((char *) FieldValue[0].value);
	*((char *) FieldValue[0].value) = '?';
	for(;;)
	{
		ret = GotoRecord(handle, ABSOLUTE, 5);
		if (ret != 5)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		TimeStamp = DBFInfo.LastUpdate;

		// cerco 
		ret =  LocateRecord(handle, NUMVALUES, &FieldValue[0], CURRENT);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != LPC_NOT_FOUND)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		if ((DBFInfo.CurrentRec != 5) || (DBFInfo.LastUpdate != TimeStamp))
			break;

		test[numtest].passed = 1;
		break;
	}
	*((char *) FieldValue[0].value) = c;
	numtest++;

// Verifica funzionalita' (2) non trovato top
	test[numtest].testid = 40309;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "LOCATE: Funzionalita'(2)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	c = *((char *) FieldValue[0].value);
	*((char *) FieldValue[1].value) = '?';
	for(;;)
	{
		ret = GotoRecord(handle, ABSOLUTE, 5);
		if (ret != 5)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		TimeStamp = DBFInfo.LastUpdate;

		// cerco
		ret =  LocateRecord(handle, NUMVALUES, &FieldValue[0], TOP);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != LPC_NOT_FOUND)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		if ((DBFInfo.CurrentRec != 5) || (DBFInfo.LastUpdate != TimeStamp))
			break;

		test[numtest].passed = 1;
		break;
	}
	*((char *) FieldValue[1].value) = c;
	numtest++;

// Verifica funzionalita' (3) non trovato wrap
	test[numtest].testid = 40310;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "LOCATE: Funzionalita'(3)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	c = *((char *) FieldValue[0].value);
	*((char *) FieldValue[2].value) = '?';
	for(;;)
	{
		ret = GotoRecord(handle, ABSOLUTE, 5);
		if (ret != 5)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		TimeStamp = DBFInfo.LastUpdate;

		// cerco
		ret =  LocateRecord(handle, NUMVALUES, &FieldValue[0], WRAP);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != LPC_NOT_FOUND)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		if ((DBFInfo.CurrentRec != 5) || (DBFInfo.LastUpdate != TimeStamp))
			break;

		test[numtest].passed = 1;
		break;
	}
	*((char *) FieldValue[2].value) = c;
	numtest++;

// Verifica funzionalita' (4)  trovato current
	test[numtest].testid = 40311;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "LOCATE: Funzionalita'(4)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = GotoRecord(handle, ABSOLUTE, 1);
		if (ret != 1)
			break;

		// cerco solo i primi 3
		ret =  LocateRecord(handle, 3, &Record20[0], CURRENT);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != 20)
			break;

		// Verifico
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		if (DBFInfo.CurrentRec != 20)
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;

// Verifica funzionalita' (5)  trovato top
	test[numtest].testid = 40312;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "LOCATE: Funzionalita'(5)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		// cerco i primi 5
		ret =  LocateRecord(handle, 5, &Record30[0], TOP);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != 30)
			break;

		// Verifico
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		if (DBFInfo.CurrentRec != 30)
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;

// Verifica funzionalita' (6) trovato wrap
	test[numtest].testid = 40313;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "LOCATE: Funzionalita'(6)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	for(;;)
	{
		ret = GotoRecord(handle, ABSOLUTE, 45);
		if (ret != 45)
			break;

		// cerco 3 di mezzo
		ret =  LocateRecord(handle, 3, &Record20[3], WRAP);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != 20)
			break;

		// Verifico
		ret = GetDBFInfo(handle, &DBFInfo);
		if (ret != LPC_OK)
			break;
		if (DBFInfo.CurrentRec != 20)
			break;

		test[numtest].passed = 1;
		break;
	}
	numtest++;
}



void TestMisto()
{
	int j, ret = 0, handle1;
	DBF_INFO_t DBFInfo;
	char c;

	printf("Test MISTO\n");

	test[numtest].testid = 40401;
	test[numtest].weight = 15;
	strcpy(test[numtest].desc, "MISTO: Funzionalita' miste");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);

	// 1) Apro il DB Prova5 e prendo informazioni da Prova4
	// 2) Vado al primo record di Prova4 
	// 3) Cerco su Prova5 dal CURRENT (EOF)
	// 4) Ciclo di lettura/scrittura: 
		// 5) Inserisco un record in fondo
		// 6) Leggo da Prova4
		// 7) Scrivo su Prova5.
	// 7bis) Verifico che i files siano uguali
	// 8) Vado al 56 su Prova4 e leggo 3 valori
	// 9) Cerco 3 valori su Prova5 dal TOP (56)
	// 10) Avanzo di 1 cerco 3 valori su Prova5 dal current (EOF)
	// 11) Vado all'1, aggiungo un 4 valore sbagliato e cerco su Prova5 con WRAP (EOF)
	// 12) Rimetto a posto il 4 e cerco tutti i  valori dal CURRENT (EOF)
	// 13) Cerco tutti i valori con WRAP (56) 

	// Il for mi serve solo come struttura per evitare il GOTO
	for(;;)
	{

		// 1
		handle1 = OpenDatabaseFile("Prova5");
		printf("Ritorno della OpenDatabaseFile: [%d]\n", ret);
		if (handle1 < 0)
			break;

		ret = GetDBFInfo(handle, &DBFInfo);
		printf("Ritorno della GetDBFInfo: [%d]\n", ret);
		if (ret != LPC_OK)
			break;

		// 2
		ret = GotoRecord(handle, ABSOLUTE, 1);
		printf("Ritorno della GotoRecord: [%d]\n", ret);
		if (ret != 1)
			break;

		// 3
		ret = LocateRecord(handle1, 3, &Record20[0], CURRENT);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != LPC_NOT_FOUND)
			break;

		// 4
		for (j = 0; j < DBFInfo.NumRecs; j++)
		{
			// 5
			ret = InsertBlankRecord(handle1, BOTTOM);
			if (ret != LPC_OK)
				break;

			// 6 
			ret = ReadRecord(handle, NUMFIELDS, &FieldValueAll[0]);
			if (ret != LPC_OK)
				break;

			// 7
			ret = EditRecord(handle1, NUMFIELDS, &FieldValueAll[0]);
			if (ret != LPC_OK)
				break;
		}

		if (ret != LPC_OK)
			break;

		// 7bis
		if (ConfrontaFiles("Prova4.dbf", "Prova5.dbf"))
			break;

		// 8
		ret = GotoRecord(handle, ABSOLUTE, 56);
		printf("Ritorno della GotoRecord: [%d]\n", ret);
		if (ret != 56)
			break;

		ret = ReadRecord(handle, 3, &FieldValue[0]);
		printf("Ritorno della ReadRecord: [%d]\n", ret);
		if (ret != LPC_OK)
			break;
		
		// 9
		ret = LocateRecord(handle1, 3, &FieldValue[0], TOP);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != 56)
			break;

		// 10
		ret = GotoRecord(handle1, RELATIVE, 1);
		printf("Ritorno della GotoRecord: [%d]\n", ret);
		if (ret != 57)
			break;

		ret = LocateRecord(handle1, 3, &FieldValue[0], CURRENT);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != LPC_NOT_FOUND)
			break;

		// 11
		ret = GotoRecord(handle1, ABSOLUTE, 1);
		printf("Ritorno della GotoRecord: [%d]\n", ret);
		if (ret != 1)
			break;

		c = *((char *) (FieldValue[3].value));
		*((char *) (FieldValue[3].value)) = '?';
		ret = LocateRecord(handle1, 4, &FieldValue[0], WRAP);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != LPC_NOT_FOUND)
			break;
		*((char *) (FieldValue[3].value)) = c;

		// 12
		ret = GotoRecord(handle1, ABSOLUTE, 57);
		printf("Ritorno della GotoRecord: [%d]\n", ret);
		if (ret != 57)
			break;

		ret = LocateRecord(handle1, 3, &FieldValue[0], WRAP);
		printf("Ritorno della LocateRecord: [%d]\n", ret);
		if (ret != 56)
			break;

		test[numtest].passed = 1;
		break;
	}
	(void) CloseDatabaseFile(handle1);
	numtest++;
}

/*********************************
* RESULT
* Stampa i risultati. Se il parametro e' NULL allora resetta i valori
**********************************/
void Result(char *Func)
{
	int i, quanti, totval;

	// Svuoto il vettore
	if (!Func)
	{
		for (i = 0; i < MAX_TEST; i++)
			memset(&test[i], 0, sizeof(TEST_t));
		numtest = 0;
		return;
	}

	// Stampo gli ID
	printf("\n");
	printf("ID:   ");
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].testid);
	}

	// Stampo i pesi e prendo il valore totale
	printf("\n");
	printf("PESO: ");
	totval = 0;
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].weight);
		totval += test[i].weight;
	}

	// Stampo i risultati
	printf("\n");
	printf("PASS: ");
	quanti = 0;
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].passed);
		if (test[i].passed)
			quanti += test[i].weight;
	}

	printf("\n\nFine Test [%s]: Numtests:[%d] Totvalue:[%d] Passed:[%d] Score:[%3.2f]\n\n", Func, numtest, totval, quanti, (double) (quanti*100/((double)totval)));
}


/*************************
* CONFRONTAFILES
**************************/
#define FILELEN 50000
int ConfrontaFiles(char *File1, char *File2)
{
	int len, len1, len2;
	char buf1[FILELEN], buf2[FILELEN];
	FILE *fp1, *fp2;

	fp1 = fopen(File1, "r");
	len1 = fread(buf1, sizeof(char), FILELEN, fp1);
	fclose(fp1);

	fp2 = fopen(File2, "r");
	len2 = fread(buf2, sizeof(char), FILELEN, fp2);
	fclose(fp2);
	if (len1 != len2)
		return(-1);

	// Azzero il timestamp
	len = DBF_NAME_LENGTH+1+sizeof(short)+sizeof(long)+sizeof(short)+sizeof(short);
	memset(buf1+len, 0, sizeof(long));
	memset(buf2+len, 0, sizeof(long));

	// Confronto i buffer
	if (memcmp(buf1, buf2, len1))
		return(-1);

	return 0;
}

/*******************
* STARTUP
********************/
int Startup(int handle, int reset)
{
	int i, idx, len, ret;

	if (IsDBFHandle(handle) == LPC_OK)
		CloseDatabaseFile(handle);

	// Se reset completo cancello, ricopio dal template e apro il DBF
	if (reset)
	{
		remove("Prova4.dbf");
		remove("Prova4.$$$");
		remove("Prova5.dbf");
		remove("Prova5.$$$");
		system("cp ProvaTest4.dbf Prova4.dbf");
		system("cp ProvaTest5.dbf Prova5.dbf");
	}
	handle = OpenDatabaseFile("Prova4");
	if (handle < 0)
		return(handle);

	ret = GetFieldInfo(handle, 0, &FieldData[0]);
	if (ret != LPC_OK)
		return(ret);

	for (i = 0; i < NUMFIELDS; i++)
	{
		// 0123456789
		// 3692581470
		idx = ((i+1)*3)%NUMFIELDS;

		// FieldValue
		strcpy(FieldValue[i].Name, FieldData[idx].Name);
		len = FieldData[idx].FieldLen;
		if (FieldData[idx].FieldType == CHARACTER)
			len++;
		FieldValue[i].value = calloc(len, sizeof(char));

		// FieldValueAll
		strcpy(FieldValueAll[i].Name, FieldData[i].Name);
		len = FieldData[i].FieldLen;
		if (FieldData[i].FieldType == CHARACTER)
			len++;
		FieldValueAll[i].value = calloc(len, sizeof(char));

		// Record20
		strcpy(Record20[i].Name, FieldData[idx].Name);
		len = FieldData[idx].FieldLen;
		if (FieldData[idx].FieldType == CHARACTER)
			len++;
		Record20[i].value = calloc(len, sizeof(char));
		// Record 30
		strcpy(Record30[i].Name, FieldData[i].Name);
		len = FieldData[i].FieldLen;
		if (FieldData[i].FieldType == CHARACTER)
			len++;
		Record30[i].value = calloc(FieldData[i].FieldLen+1, sizeof(char));
	}

// Riempio FieldValue
	Nome = (char *) FieldValue[9].value;
	Eta = (double *) FieldValue[6].value;
	Data = (time_t *) FieldValue[3].value;
	Reddito = (double *) FieldValue[0].value;
	Studente = (char *) FieldValue[7].value;
	Sesso = (char *) FieldValue[4].value;
	Comune = (char *) FieldValue[1].value;
	Provincia = (char *) FieldValue[8].value;
	Telefono = (char *) FieldValue[5].value;
	Indirizzo = (char *) FieldValue[2].value;

	memset(Nome, 'R', 127);
	memset(Nome+127, 0, 30);
	memset(Nome+157, 'Z', 99);
	*Eta = 10;
	*Data = 777;
	*Reddito = 0.87;
	*Studente = (char) 0;
	*Sesso = 'F';
	strcpy(Comune, "HELLZAPOPPING");
	strcpy(Provincia, "QQ");
	strcpy(Telefono, "00112233");
	strcpy(Indirizzo, "Via Alvise Persichetti 77");
	memset(Indirizzo+strlen(Indirizzo), 7, 3);

// Riempio il Record 20
	Nome = (char *) Record20[9].value;
	Eta = (double *) Record20[6].value;
	Data = (time_t *) Record20[3].value;
	Reddito = (double *) Record20[0].value;
	Studente = (char *) Record20[7].value;
	Sesso = (char *) Record20[4].value;
	Comune = (char *) Record20[1].value;
	Provincia = (char *) Record20[8].value;
	Telefono = (char *) Record20[5].value;
	Indirizzo = (char *) Record20[2].value;

	memset(Nome, 'A', 127);
	Nome[127] = (char) NULL;
	memset(Nome+128, 'Z', 127);
	*Eta = 23;
	*Data = 1234567;
	*Reddito = 44.55;
	*Studente = (char) 1;
	*Sesso = 'M';
	strcpy(Comune, "ROMA");
	strcpy(Provincia, "RM");
	strcpy(Telefono, "060606");
	strcpy(Indirizzo, "Via delle Foglie Gialle 37");

// Riempio il Record30
	Nome = (char *) Record30[0].value;
	Eta = (double *) Record30[1].value;
	Data = (time_t *) Record30[2].value;
	Reddito = (double *) Record30[3].value;
	Studente = (char *) Record30[4].value;
	Sesso = (char *) Record30[5].value;
	Comune = (char *) Record30[6].value;
	Provincia = (char *) Record30[7].value;
	Telefono = (char *) Record30[8].value;
	Indirizzo = (char *) Record30[9].value;

	memset(Nome, 'b', 10);
	memset(Nome+10, 0, 100);
	memset(Nome+110, 7, 100);
	memset(Nome+210, 'j', 45);
	*Eta = 99;
	*Data = 7654321;
	*Reddito = 123.456;
	*Studente = (char) 0;
	*Sesso = 'F';
	strcpy(Comune, "COPURNO ASSIANO");
	strcpy(Provincia, "GH");
	strcpy(Telefono, "338338338");
	strcpy(Indirizzo, "Piazza delle venticinque palme");

	return(ret);
}

/*****************
* QUIT
*****************/
void Quit(void)
{
	int i;

	(void) CloseDatabaseFile(handle);

	for (i = 0; i < NUMFIELDS; i++)
	{
		free(FieldValue[i].value);
		free(FieldValueAll[i].value);
		free(Record20[i].value);
		free(Record30[i].value);
	}

}

/*********************
* CONFRONTAVALUES
**********************/
int ConfrontaValues(FIELD_VALUE_t *FieldValue, FIELD_VALUE_t *FieldValueRef, int NumValues)
{
	int i, j, k;

	for (i = 0; i < NumValues; i++)
	{
		for (j = 0; j < NUMFIELDS; j++)
			if (!strcmp(FieldValue[i].Name, FieldData[j].Name))
				break; 
		for (k = 0; k < NUMFIELDS; k++)
			if (!strcmp(FieldValue[i].Name, FieldValueRef[k].Name))
				break; 
		if (memcmp(FieldValue[i].value, FieldValueRef[k].value, FieldData[j].FieldLen))
			break;
	}

	if (i < NumValues)
		return(-1);

	return(0);
}

/*! \} */	// Fine gruppo TestModulo4
/*! \} */	// Fine gruppo Modulo4
